# -*- coding: utf-8 -*-

import logging

from odoo.addons.saas_worker.util import ip2coordinates

_logger = logging.getLogger(__name__)

##################################################################################################
# Country list and hosting
##################################################################################################

# Use the continent hierarchy defined by Google (mostly based on the one defined by the UN).
# This allows us to easily assign a large portion of a continent to a hosting region.
# See:
# https://developers.google.com/chart/interactive/docs/gallery/geochart#continent-hierarchy-and-codes
# https://unstats.un.org/unsd/methodology/m49/

SUBCONT_COUNTRY = {
    "Northern Africa": ["DZ", "EG", "EH", "LY", "MA", "SD", "SS", "TN"],
    "Western Africa": [
        "BF", "BJ", "CI", "CV", "GH", "GM", "GN", "GW", "LR",
        "ML", "MR", "NE", "NG", "SH", "SL", "SN", "TG",
    ],
    "Middle Africa": ["AO", "CD", "ZR", "CF", "CG", "CM", "GA", "GQ", "ST", "TD"],
    "Eastern Africa": [
        "BI", "DJ", "ER", "ET", "KE", "KM", "MG", "MU", "MW", "MZ",
        "RE", "RW", "SC", "SO", "TZ", "UG", "YT", "ZM", "ZW",
    ],
    "Southern Africa": ["BW", "LS", "NA", "SZ", "ZA"],
    "Northern Europe": [
        "GG", "JE", "AX", "DK", "EE", "FI", "FO", "GB",
        "IE", "IM", "IS", "LT", "LV", "NO", "SE", "SJ",
    ],
    "Western Europe": ["AT", "BE", "CH", "DE", "DD", "FR", "FX", "LI", "LU", "MC", "NL"],
    "Eastern Europe": ["BG", "BY", "CZ", "HU", "MD", "PL", "RO", "RU", "SU", "SK", "UA"],
    "Southern Europe": [
        "AD", "AL", "BA", "ES", "GI", "GR", "HR", "IT", "ME",
        "MK", "MT", "RS", "PT", "SI", "SM", "VA", "YU",
    ],
    "Northern America": ["BM", "CA", "GL", "PM", "US"],
    "Caribbean": [
        "AG", "AI", "AN", "AW", "BB", "BL", "BS", "CU", "DM", "DO", "GD", "GP", "HT",
        "JM", "KN", "KY", "LC", "MF", "MQ", "MS", "PR", "TC", "TT", "VC", "VG", "VI",
    ],
    "Central America": ["BZ", "CR", "GT", "HN", "MX", "NI", "PA", "SV"],
    "South America": [
        "AR", "BO", "BR", "CL", "CO", "EC", "FK", "GF", "GY",
        "PE", "PY", "SR", "UY", "VE",
    ],
    "Central Asia": ["TM", "TJ", "KG", "KZ", "UZ"],
    "Eastern Asia": ["CN", "HK", "JP", "KP", "KR", "MN", "MO", "TW"],
    "Southern Asia": ["AF", "BD", "BT", "IN", "IR", "LK", "MV", "NP", "PK"],
    "South-Eastern Asia": [
        "BN", "ID", "KH", "LA", "MM", "BU", "MY",
        "PH", "SG", "TH", "TL", "TP", "VN",
    ],
    "Western Asia": [
        "AE", "AM", "AZ", "BH", "CY", "GE", "IL", "IQ", "JO", "KW",
        "LB", "OM", "PS", "QA", "SA", "NT", "SY", "TR", "YE", "YD",
    ],
    "Australia and N-Z": ["AU", "NF", "NZ"],
    "Melanesia": ["FJ", "NC", "PG", "SB", "VU"],
    "Micronesia": ["FM", "GU", "KI", "MH", "MP", "NR", "PW"],
    "Polynesia": ["AS", "CK", "NU", "PF", "PN", "TK", "TO", "TV", "WF", "WS"],
}

HOSTING_SUBCONT = {
    "am": [
        "Northern America",
        "Caribbean",
        "Central America",
        "Polynesia",
        "South America",
    ],
    "as": [
        "Eastern Asia",
        "South-Eastern Asia",
    ],
    "au": [
        "Australia and N-Z",
        "Melanesia",
        "Micronesia",
    ],
    "me": [
        "Eastern Africa",
        "Central Asia",
        "Southern Asia",
        "Western Asia",
    ],
    "eu": [
        "Northern Africa",
        "Western Africa",
        "Middle Africa",
        "Southern Africa",
        "Northern Europe",
        "Western Europe",
        "Eastern Europe",
        "Southern Europe",
    ],
}


COUNTRY_SUBCONT = {v: k for k, l in SUBCONT_COUNTRY.items() for v in l}
SUBCONT_HOSTING = {v: k for k, l in HOSTING_SUBCONT.items() for v in l}

# For future changes: make sure a sub-continent is part of a single region.
assert len(SUBCONT_COUNTRY) == len([v for sl in HOSTING_SUBCONT.values() for v in sl])
assert set(COUNTRY_SUBCONT.values()) == set(SUBCONT_HOSTING.keys())

# Generate the {county_code: region} mapping
HOSTING_MAP = {cc: SUBCONT_HOSTING[cont] for cc, cont in COUNTRY_SUBCONT.items()}

# Add exceptions below
HOSTING_MAP["CU"] = "eu"
HOSTING_MAP["IQ"] = "eu"
HOSTING_MAP["AE"] = "sa"
HOSTING_MAP["SA"] = "sa"

# Only declare if corresponding servers exist
HOSTING_PREFIX_MAP = {
    "edu-xx-": {},
    "edu-": {
        "am": "amedu",
        "as": "asedu",
        "au": "asedu",
        "eu": "euedu",
        "me": "meedu",
        "sa": "meedu",
    }
}


def ip2hosting(ip):
    loc = ip2coordinates(ip)
    _logger.info(
        "GeoIP resolution for IP %s -> %s", ip, loc and loc.get("country_code", "n/a") or "no loc"
    )
    if loc:
        return HOSTING_MAP.get(loc["country_code"], "eu")
    return "eu"


def name_based_hosting(hosting, name):
    for prefix in HOSTING_PREFIX_MAP.keys():
        if name.startswith(prefix):
            return HOSTING_PREFIX_MAP[prefix].get(hosting, hosting)
